@@ -1,13 +1,15 @@
 import knowledge_clustering.diagnose as diag
-import knowledge_clustering.kltex as kltex
 import knowledge_clustering.clustering as clust
 import knowledge_clustering.config as config
+import knowledge_clustering.knowledges as kls
+import knowledge_clustering.tex_code as tex
 import knowledge_clustering.scope_meaning as sm
 import knowledge_clustering.file_updater as fu
 import knowledge_clustering.add_quotes as quotes
 import knowledge_clustering.add_AP as ap
 
 
+import copy
 import click
 from click_default_group import DefaultGroup
 import nltk
@@ -46,6 +48,7 @@ def init():
 @click.option(
     "--knowledge",
     "-k",
+    "kl_file",
     type=click.Path(
         exists=True, file_okay=True, dir_okay=False, writable=True, readable=True
     ),
@@ -55,6 +58,7 @@ def init():
 @click.option(
     "--diagnose",
     "-d",
+    "dg_file",
     type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True),
     help="Diagnose file produced by LaTeX.",
     required=True,
@@ -80,59 +84,49 @@ the possible meaning of those scope inferred by knowledge-clustering.",
     help=f"Specify the configuration file. By default the configuration file in the folder {CONFIG_DIR} \
 corresponding to your language is used.",
 )
-def cluster(knowledge, diagnose, scope, lang, config_file):
+def cluster(kl_file, dg_file, scope, lang, config_file):
     """
     Edit a KNOWLEDGE file using the knowledges present
     in a DIAGNOSE file.
     """
-    with open(knowledge, "r") as f:
-        document, known_knowledges = kltex.parse(f)
+    kl = kls.Knowledges(kl_file)
 
     if config_file == None:
         config_file = CONFIG_FILE[lang]
 
     list_prefixes = config.parse(config_file)
 
-    scopes_meaning = sm.inferAllScopes(known_knowledges, NLTK_LANG[lang])
+    scopes_meaning = sm.inferAllScopes(kl.get_all_bags(), NLTK_LANG[lang])
     if scope:
         sm.printScopes(scopes_meaning, print_meaning=True)
 
-    unknown_knowledges = diag.parse(diagnose)
+    unknown_knowledges = diag.parse(dg_file)
 
     if len(unknown_knowledges) == 0:
         return
 
-    len_known_knowledges = len(known_knowledges)
-    len_bags = [len(bag) for bag in known_knowledges]
-    # update known_knowledges using the clustering algorithm
+    # update `kl` using the clustering algorithm
     clust.clustering(
-        known_knowledges,
+        kl,
         unknown_knowledges,
         ALPHA,
         list_prefixes,
         scopes_meaning,
         NLTK_LANG[lang],
     )
-    # Compute updated_knowledges and new_knowledges
-    new_knowledges = known_knowledges[len_known_knowledges:]
-    updated_knowledges = [
-        known_knowledges[bag_id][len_bags[bag_id] :]
-        for bag_id in range(len_known_knowledges)
-    ]
     print(
-        f"Found a solution by adding {len(new_knowledges)} new knowledge"
-        + ("s" if len(new_knowledges) >= 2 else "")
+        f"Found a solution by adding {len(kl.get_new_bags())} new bag"
+        + ("s" if len(kl.get_new_bags()) >= 2 else "")
         + "."
     )
-
-    with fu.AtomicUpdate(knowledge) as f:
-        kltex.writeDocument(f, document, updated_knowledges, new_knowledges)
+    kl.write_knowledges_in_file()
 
 
 @cli.command()
 @click.option(
     "--tex",
     "-t",
+    "tex_file",
     type=click.Path(
         exists=True, file_okay=True, dir_okay=False, writable=True, readable=True
     ),
@@ -142,6 +136,7 @@ def cluster(knowledge, diagnose, scope, lang, config_file):
 @click.option(
     "--knowledge",
     "-k",
+    "kl_file",
     type=click.Path(
         exists=True, file_okay=True, dir_okay=False, writable=True, readable=True
     ),
@@ -156,28 +151,23 @@ def cluster(knowledge, diagnose, scope, lang, config_file):
     default=1,
     help="When finding a match, number of lines (preceding the match) that are printed in the prompt to the user.",
 )
-def addquotes(tex, knowledge, print_line):
+def addquotes(tex_file, kl_file, print_line):
     """
     Finds knowledges defined in KNOWLEDGE that appear in TEX without quote
     symbols. Proposes to add quotes around them.
     """
-    with open(tex, "r") as f:
-        tex_document = f.read()
-    with open(knowledge, "r") as f:
-        kl_document, known_knowledges = kltex.parse(f)
-    known_knowledges_flat = [kl for bag in known_knowledges for kl in bag]
+    tex_hash = fu.hash_file(tex_file)
+    with open(tex_file, "r") as f:
+        tex_code = tex.TexCode(f.read())
+    kl = kls.Knowledges(kl_file)
     tex_document_new, new_knowledges = quotes.quote_maximal_substrings(
-        tex_document, known_knowledges_flat, print_line, size_tab=4
+        tex_code, kl, print_line, size_tab=4
     )
-    with fu.AtomicUpdate(tex) as f:
+    with fu.AtomicUpdate(tex_file, original_hash=tex_hash) as f:
         f.write(tex_document_new)
-    updated_knowledges = [
-        [new_kl for (def_kl, new_kl) in new_knowledges if def_kl in bag]
-        for bag in known_knowledges
-    ]
-    if updated_knowledges != []:
-        with fu.AtomicUpdate(knowledge) as f:
-            kltex.writeDocument(f, kl_document, updated_knowledges, [], nocomment=True)
+    for known_kl, new_kl in new_knowledges:
+        kl.define_synonym_of(new_kl, known_kl)
+    kl.write_knowledges_in_file(nocomment=True)
 
 
 @cli.command()
